@@ -18,11 +18,11 @@ n=500 # n nodes
 p=0.1
 w=[p*n for i in range(n)] # w = p*n for all nodes
 G=expected_degree_graph(w)  # configuration model
-print "Degree histogram"
-print "degree (#nodes) ****"
+print("Degree histogram")
+print("degree (#nodes) ****")
 dh=degree_histogram(G)
 low=min(degree(G))
 for i in range(low,len(dh)):
     bar=''.join(dh[i]*['*'])
-    print "%2s (%2s) %s"%(i,dh[i],bar)
+    print("%2s (%2s) %s"%(i,dh[i],bar))
 
