@@ -15,10 +15,10 @@ Using
     >>> import networkx as nx
     >>> G=nx.Graph()
     >>> G.add_edge(1,2)
-    >>> G.add_node("spam")
-    >>> print(G.nodes())
-    [1, 2, 'spam']
-    >>> print(G.edges())
+    >>> G.add_node(42)
+    >>> print(sorted(G.nodes()))
+    [1, 2, 42]
+    >>> print(sorted(G.edges()))
     [(1, 2)]
 """
 #    Copyright (C) 2004-2010 by
@@ -50,25 +50,35 @@ __license__  = release.license
 __date__ = release.date
 __version__ = release.version
 
-# these packages work with Python >= 2.6
+#These are import orderwise
 from networkx.exception import  *
+import networkx.external
+import networkx.utils
+# these packages work with Python >= 2.6
+
 import networkx.classes
 from networkx.classes import *
+
+
 import networkx.convert
 from networkx.convert import *
+
 import networkx.relabel
 from networkx.relabel import *
+
 import networkx.generators
 from networkx.generators import *
-from networkx.readwrite import *
+
 import networkx.readwrite
+from networkx.readwrite import *
+
 #Need to test with SciPy, when available
 import networkx.algorithms
 from networkx.algorithms import *
 import networkx.linalg
+
 from networkx.linalg import *
 from networkx.tests.test import run as test
-import networkx.utils
 
 import networkx.drawing
 from networkx.drawing import *
