@@ -6,17 +6,18 @@ from networkx.algorithms.cluster import *
 from networkx.algorithms.clique import *
 from networkx.algorithms.community import *
 from networkx.algorithms.components import *
+from networkx.algorithms.connectivity import *
 from networkx.algorithms.core import *
 from networkx.algorithms.cycles import *
 from networkx.algorithms.dag import *
 from networkx.algorithms.distance_measures import *
 from networkx.algorithms.flow import *
+from networkx.algorithms.hierarchy import *
 from networkx.algorithms.matching import *
 from networkx.algorithms.mis import *
 from networkx.algorithms.mst import *
 from networkx.algorithms.link_analysis import *
 from networkx.algorithms.operators import *
-from networkx.algorithms.product import *
 from networkx.algorithms.shortest_paths import *
 from networkx.algorithms.smetric import *
 from networkx.algorithms.traversal import *
@@ -28,6 +29,7 @@ from networkx.algorithms.richclub import *
 from networkx.algorithms.distance_regular import *
 from networkx.algorithms.swap import *
 from networkx.algorithms.graphical import *
+from networkx.algorithms.simple_paths import *
 
 import networkx.algorithms.assortativity
 import networkx.algorithms.bipartite
@@ -35,12 +37,14 @@ import networkx.algorithms.centrality
 import networkx.algorithms.cluster
 import networkx.algorithms.clique
 import networkx.algorithms.components
+import networkx.algorithms.connectivity
 import networkx.algorithms.flow
 import networkx.algorithms.isomorphism
 import networkx.algorithms.link_analysis
 import networkx.algorithms.shortest_paths
 import networkx.algorithms.traversal
 import networkx.algorithms.chordal
+import networkx.algorithms.operators
 
 from networkx.algorithms.bipartite import projected_graph,project,is_bipartite
 from networkx.algorithms.isomorphism import is_isomorphic,could_be_isomorphic,\
