@@ -4,17 +4,95 @@
 Bipartite Graph Algorithms
 ==========================
 """
-import networkx as nx
-__author__ = """Aric Hagberg (hagberg@lanl.gov)"""
-#    Copyright (C) 2011 by 
+#    Copyright (C) 2012 by
 #    Aric Hagberg <hagberg@lanl.gov>
 #    Dan Schult <dschult@colgate.edu>
 #    Pieter Swart <swart@lanl.gov>
 #    All rights reserved.
 #    BSD license.
-__all__ = ['is_bipartite', 'is_bipartite_node_set',
-           'color', 'sets',
-           'density', 'degrees']
+import networkx as nx
+from itertools import count
+__author__ = """\n""".join(['Jordi Torrents <jtorrents@milnou.net>',
+                            'Aric Hagberg <aric.hagberg@gmail.com>'])
+__all__ = [ 'is_bipartite',
+            'is_bipartite_node_set',
+            'color',
+            'sets',
+            'density',
+            'degrees',
+            'biadjacency_matrix']
+
+def biadjacency_matrix(G, row_order, column_order=None,
+                            weight='weight', dtype=None):
+    r"""Return the biadjacency matrix of the bipartite graph G.
+
+    Let `G = (U, V, E)` be a bipartite graph with node sets
+    `U = u_{1},...,u_{r}` and `V = v_{1},...,v_{s}`. The biadjacency
+    matrix [1] is the `r` x `s` matrix `B` in which `b_{i,j} = 1`
+    if, and only if, `(u_i, v_j) \in E`. If the parameter `weight` is
+    not `None` and matches the name of an edge attribute, its value is
+    used instead of 1.
+
+    Parameters
+    ----------
+    G : graph
+       A NetworkX graph
+
+    row_order : list of nodes
+       The rows of the matrix are ordered according to the list of nodes.
+
+    column_order : list, optional
+       The columns of the matrix are ordered according to the list of nodes.
+       If column_order is None, then the ordering of columns is arbitrary.
+
+    weight : string or None, optional (default='weight')
+       The edge data key used to provide each value in the matrix.
+       If None, then each edge has weight 1.
+
+    dtype : NumPy data type, optional
+        A valid single NumPy data type used to initialize the array.
+        This must be a simple type such as int or numpy.float64 and
+        not a compound data type (see to_numpy_recarray)
+        If None, then the NumPy default is used.
+
+    Returns
+    -------
+    B : numpy matrix
+      Biadjacency matrix representation of the bipartite graph G.
+
+    Notes
+    -----
+    No attempt is made to check that the input graph is bipartite.
+
+    For directed bipartite graphs only successors are considered as neighbors.
+    To obtain an adjacency matrix with ones (or weight values) for both
+    predecessors and successors you have to generate two biadjacency matrices
+    where the rows of one of them are the columns of the other, and then add
+    one to the transpose of the other.
+
+    See Also
+    --------
+    to_numpy_matrix
+    adjacency_matrix
+
+    References
+    ----------
+    [1] http://en.wikipedia.org/wiki/Adjacency_matrix#Adjacency_matrix_of_a_bipartite_graph
+    """
+    try:
+        import numpy as np
+    except ImportError:
+        raise ImportError('adjacency_matrix() requires numpy ',
+                          'http://scipy.org/')
+    if column_order is None:
+        column_order = list(set(G) - set(row_order))
+    row = dict(zip(row_order,count()))
+    col = dict(zip(column_order,count()))
+    M = np.zeros((len(row),len(col)), dtype=dtype)
+    for u in row_order:
+        for v, d in G[u].items():
+            M[row[u],col[v]] = d.get(weight, 1)
+    return np.asmatrix(M)
 
 def color(G):
     """Returns a two-coloring of the graph.
@@ -23,7 +101,7 @@ def color(G):
 
     Parameters
     ----------
-    G : NetworkX graph 
+    G : NetworkX graph
 
     Returns
     -------
@@ -43,7 +121,7 @@ def color(G):
     {0: 1, 1: 0, 2: 1, 3: 0}
 
     You can use this to set a node attribute indicating the biparite set:
-    
+
     >>> nx.set_node_attributes(G, 'bipartite', c)
     >>> print(G.node[0]['bipartite'])
     1
@@ -57,18 +135,18 @@ def color(G):
                                                   G.successors_iter(v)])
     else:
         neighbors=G.neighbors_iter
-    
+
     color = {}
     for n in G: # handle disconnected graphs
         if n in color or len(G[n])==0: # skip isolates
             continue
-        queue = [n]  
+        queue = [n]
         color[n] = 1 # nodes seen with color (1 or 0)
         while queue:
             v = queue.pop()
             c = 1 - color[v] # opposite color of node v
             for w in neighbors(v):
-                if w in color: 
+                if w in color:
                     if color[w] == color[v]:
                         raise nx.NetworkXError("Graph is not bipartite.")
                 else:
@@ -83,7 +161,7 @@ def is_bipartite(G):
 
     Parameters
     ----------
-    G : NetworkX graph 
+    G : NetworkX graph
 
     Examples
     --------
@@ -101,13 +179,13 @@ def is_bipartite(G):
         return True
     except nx.NetworkXError:
         return False
-    
+
 def is_bipartite_node_set(G,nodes):
     """Returns True if nodes and G/nodes are a bipartition of G.
 
     Parameters
     ----------
-    G : NetworkX graph 
+    G : NetworkX graph
 
     nodes: list or container
       Check if nodes are a one of a bipartite set.
@@ -128,11 +206,11 @@ def is_bipartite_node_set(G,nodes):
     S=set(nodes)
     for CC in nx.connected_component_subgraphs(G):
         X,Y=sets(CC)
-        if not ( (X.issubset(S) and Y.isdisjoint(S)) or 
+        if not ( (X.issubset(S) and Y.isdisjoint(S)) or
                  (Y.issubset(S) and X.isdisjoint(S)) ):
             return False
     return True
-    
+
 
 def sets(G):
     """Returns bipartite node sets of graph G.
@@ -141,7 +219,7 @@ def sets(G):
 
     Parameters
     ----------
-    G : NetworkX graph 
+    G : NetworkX graph
 
     Returns
     -------
@@ -172,7 +250,7 @@ def density(B, nodes):
 
     Parameters
     ----------
-    G : NetworkX graph 
+    G : NetworkX graph
 
     nodes: list or container
       Nodes in one set of the bipartite graph.
@@ -215,7 +293,7 @@ def degrees(B, nodes, weight=None):
 
     Parameters
     ----------
-    G : NetworkX graph 
+    G : NetworkX graph
 
     nodes: list or container
       Nodes in one set of the bipartite graph.
@@ -246,3 +324,12 @@ def degrees(B, nodes, weight=None):
     bottom=set(nodes)
     top=set(B)-bottom
     return (B.degree(top,weight),B.degree(bottom,weight))
+
+
+# fixture for nose tests
+def setup_module(module):
+    from nose import SkipTest
+    try:
+        import numpy
+    except:
+        raise SkipTest("NumPy not available")
