@@ -6,6 +6,7 @@ from networkx.algorithms.centrality.current_flow_betweenness import *
 from networkx.algorithms.centrality.current_flow_betweenness_subset import *
 from networkx.algorithms.centrality.degree_alg import *
 from networkx.algorithms.centrality.eigenvector import *
+from networkx.algorithms.centrality.katz import *
 from networkx.algorithms.centrality.load import *
 from networkx.algorithms.centrality.communicability_alg import *
 import networkx.algorithms.centrality.betweenness
@@ -16,4 +17,4 @@ import networkx.algorithms.centrality.degree_alg
 import networkx.algorithms.centrality.eigenvector
 import networkx.algorithms.centrality.load
 import networkx.algorithms.centrality.communicability_alg
-
+import networkx.algorithms.centrality.katz
