@@ -7,16 +7,18 @@ Betweenness centrality measures.
 #    Pieter Swart <swart@lanl.gov>
 #    All rights reserved.
 #    BSD license.
+import heapq
+import networkx as nx
+import random
 __author__ = """Aric Hagberg (hagberg@lanl.gov)"""
 
 __all__ = ['betweenness_centrality',
            'edge_betweenness_centrality',
            'edge_betweenness']
 
-import heapq
-import networkx as nx
-
-def betweenness_centrality(G, normalized=True, weight=None, endpoints=False):
+def betweenness_centrality(G, k=None, normalized=True, weight=None, 
+                           endpoints=False, 
+                           seed=None):
     r"""Compute the shortest-path betweenness centrality for nodes.
 
     Betweenness centrality of a node `v` is the sum of the
@@ -37,13 +39,18 @@ def betweenness_centrality(G, normalized=True, weight=None, endpoints=False):
     G : graph
       A NetworkX graph 
 
-    normalized : bool, optional  
-      If True the betweenness values are normalized by
-      `1/(n-1)(n-2)` where `n` is the number of nodes in G.
+    k : int, optional (default=None)
+      If k is not None use k node samples to estimate betweenness.
+      The value of k <= n where n is the number of nodes in the graph.
+      Higher values give better approximation. 
 
-    weight : None, True or string, optional  
+    normalized : bool, optional  
+      If True the betweenness values are normalized by `2/((n-1)(n-2))` 
+      for graphs, and `1/((n-1)(n-2))` for directed graphs where `n` 
+      is the number of nodes in G.
+
+    weight : None or string, optional  
       If None, all edge weights are considered equal.
-      If True, edge attribute 'weight' is used as weight of each edge.
       Otherwise holds the name of the edge attribute used as weight.
 
     endpoints : bool, optional  
@@ -64,6 +71,10 @@ def betweenness_centrality(G, normalized=True, weight=None, endpoints=False):
     The algorithm is from Ulrik Brandes [1]_.
     See [2]_ for details on algorithms for variations and related metrics.
 
+    For approximate betweenness calculations set k=#samples to use 
+    k nodes ("pivots") to estimate the betweenness values. For an estimate
+    of the number of pivots needed see [3]_.
+
     For weighted graphs the edge weights must be greater than zero.
     Zero edge weights can produce an infinite number of equal length 
     paths between pairs of nodes.
@@ -78,25 +89,34 @@ def betweenness_centrality(G, normalized=True, weight=None, endpoints=False):
        Centrality and their Generic Computation. 
        Social Networks 30(2):136-145, 2008.
        http://www.inf.uni-konstanz.de/algo/publications/b-vspbc-08.pdf
+    .. [3] Ulrik Brandes and Christian Pich: 
+       Centrality Estimation in Large Networks. 
+       International Journal of Bifurcation and Chaos 17(7):2303-2318, 2007.
+       http://www.inf.uni-konstanz.de/algo/publications/bp-celn-06.pdf
     """
     betweenness=dict.fromkeys(G,0.0) # b[v]=0 for v in G
-    for s in G:
+    if k is None:
+        nodes = G
+    else:
+        random.seed(seed)
+        nodes = random.sample(G.nodes(), k)
+    for s in nodes:
         # single source shortest paths
         if weight is None:  # use BFS
             S,P,sigma=_single_source_shortest_path_basic(G,s)
         else:  # use Dijkstra's algorithm
-            if weight is True: weight='weight'
             S,P,sigma=_single_source_dijkstra_path_basic(G,s,weight)
         # accumulation
-        if endpoints: 
-            betweenness=_accumulate_endpoints(betweenness,S,P,sigma,s)        
+        if endpoints:
+            betweenness=_accumulate_endpoints(betweenness,S,P,sigma,s)
         else:
             betweenness=_accumulate_basic(betweenness,S,P,sigma,s)
     # rescaling
-    betweenness=_rescale(betweenness,
+    betweenness=_rescale(betweenness, len(G),
                          normalized=normalized,
-                         directed=G.is_directed())
-    return betweenness            
+                         directed=G.is_directed(),
+                         k=k)
+    return betweenness
 
 
 def edge_betweenness_centrality(G,normalized=True,weight=None):
@@ -119,12 +139,12 @@ def edge_betweenness_centrality(G,normalized=True,weight=None):
       A NetworkX graph 
 
     normalized : bool, optional
-      If True the betweenness values are normalized by 
-      `1/(n-1)(n-2)` where `n` is the number of nodes in G.
+      If True the betweenness values are normalized by `2/(n(n-1))` 
+      for graphs, and `1/(n(n-1))` for directed graphs where `n` 
+      is the number of nodes in G.
        
-    weight : None, True or string, optional  
+    weight : None or string, optional  
       If None, all edge weights are considered equal.
-      If True, edge attribute 'weight' is used as weight of each edge.
       Otherwise holds the name of the edge attribute used as weight.
 
     Returns
@@ -157,23 +177,22 @@ def edge_betweenness_centrality(G,normalized=True,weight=None):
     """
     betweenness=dict.fromkeys(G,0.0) # b[v]=0 for v in G
     # b[e]=0 for e in G.edges()
-    betweenness.update(dict.fromkeys(G.edges(),0.0)) 
+    betweenness.update(dict.fromkeys(G.edges(),0.0))
     for s in G:
         # single source shortest paths
         if weight is None:  # use BFS
             S,P,sigma=_single_source_shortest_path_basic(G,s)
         else:  # use Dijkstra's algorithm
-            if weight is True: weight='weight'
             S,P,sigma=_single_source_dijkstra_path_basic(G,s,weight)
         # accumulation
         betweenness=_accumulate_edges(betweenness,S,P,sigma,s)
     # rescaling
     for n in G: # remove nodes to only return edges 
         del betweenness[n]
-    betweenness=_rescale(betweenness,
-                         normalized=normalized,
-                         directed=G.is_directed())
-    return betweenness            
+    betweenness=_rescale_e(betweenness, len(G),
+                           normalized=normalized,
+                           directed=G.is_directed())
+    return betweenness
 
 # obsolete name
 def edge_betweenness(G,normalized=True,weight=None):
@@ -219,10 +238,10 @@ def _single_source_dijkstra_path_basic(G,s,weight='weight'):
     sigma[s]=1.0
     push=heapq.heappush
     pop=heapq.heappop
-    seen = {s:0} 
+    seen = {s:0}
     Q=[]   # use Q as heap with (distance,node id) tuples
     push(Q,(0,s,s))
-    while Q:   
+    while Q:
         (dist,pred,v)=pop(Q)
         if v in D:
             continue # already searched this node.
@@ -242,7 +261,7 @@ def _single_source_dijkstra_path_basic(G,s,weight='weight'):
     return S,P,sigma
 
 def _accumulate_basic(betweenness,S,P,sigma,s):
-    delta=dict.fromkeys(S,0) 
+    delta=dict.fromkeys(S,0)
     while S:
         w=S.pop()
         coeff=(1.0+delta[w])/sigma[w]
@@ -254,7 +273,7 @@ def _accumulate_basic(betweenness,S,P,sigma,s):
 
 def _accumulate_endpoints(betweenness,S,P,sigma,s):
     betweenness[s]+=len(S)-1
-    delta=dict.fromkeys(S,0) 
+    delta=dict.fromkeys(S,0)
     while S:
         w=S.pop()
         coeff=(1.0+delta[w])/sigma[w]
@@ -265,7 +284,7 @@ def _accumulate_endpoints(betweenness,S,P,sigma,s):
     return betweenness
 
 def _accumulate_edges(betweenness,S,P,sigma,s):
-    delta=dict.fromkeys(S,0) 
+    delta=dict.fromkeys(S,0)
     while S:
         w=S.pop()
         coeff=(1.0+delta[w])/sigma[w]
@@ -280,13 +299,30 @@ def _accumulate_edges(betweenness,S,P,sigma,s):
             betweenness[w]+=delta[w]
     return betweenness
 
-def _rescale(betweenness,normalized,directed=False):
+def _rescale(betweenness,n,normalized,directed=False,k=None):
     if normalized is True:
-        order=len(betweenness)
-        if order <=2:
+        if n <=2:
             scale=None  # no normalization b=0 for all nodes
         else:
-            scale=1.0/((order-1)*(order-2))
+            scale=1.0/((n-1)*(n-2))
+    else: # rescale by 2 for undirected graphs
+        if not directed:
+            scale=1.0/2.0
+        else:
+            scale=None
+    if scale is not None:
+        if k is not None:
+            scale=scale*n/k
+        for v in betweenness:
+            betweenness[v] *= scale
+    return betweenness
+
+def _rescale_e(betweenness,n,normalized,directed=False):
+    if normalized is True:
+        if n <=1:
+            scale=None  # no normalization b=0 for all nodes
+        else:
+            scale=1.0/(n*(n-1))
     else: # rescale by 2 for undirected graphs
         if not directed:
             scale=1.0/2.0
@@ -296,4 +332,3 @@ def _rescale(betweenness,normalized,directed=False):
         for v in betweenness:
             betweenness[v] *= scale
     return betweenness
-
