@@ -1,8 +1,19 @@
 #!/usr/bin/env python
 from nose.tools import *
+from nose import SkipTest
 import networkx
 
-class TestFlowClosenessCentrality():
+class TestFlowClosenessCentrality(object):
+    numpy=1 # nosetests attribute, use nosetests -a 'not numpy' to skip test
+    @classmethod
+    def setupClass(cls):
+        global np
+        try:
+            import numpy as np
+            import scipy
+        except ImportError:
+            raise SkipTest('NumPy not available.')
+        
         
     def test_K4(self):
         """Closeness centrality: K4"""
@@ -41,5 +52,5 @@ class TestFlowClosenessCentrality():
 
 
 
-class TestWeightedFlowClosenessCentrality():
+class TestWeightedFlowClosenessCentrality(object):
     pass
