@@ -1,13 +1,11 @@
-"""
-PageRank analysis of graph structure.
-"""
-#    Copyright (C) 2004-2010 by 
+"""PageRank analysis of graph structure. """
+#    Copyright (C) 2004-2011 by
 #    Aric Hagberg <hagberg@lanl.gov>
 #    Dan Schult <dschult@colgate.edu>
 #    Pieter Swart <swart@lanl.gov>
 #    All rights reserved.
 #    BSD license.
-#    NetworkX:http://networkx.lanl.gov/. 
+#    NetworkX:http://networkx.lanl.gov/
 import networkx as nx
 from networkx.exception import NetworkXError
 __author__ = """Aric Hagberg (hagberg@lanl.gov)"""
@@ -20,16 +18,16 @@ def pagerank(G,alpha=0.85,personalization=None,
     PageRank computes a ranking of the nodes in the graph G based on
     the structure of the incoming links. It was originally designed as
     an algorithm to rank web pages.
-    
+
     Parameters
     -----------
     G : graph
-      A NetworkX graph 
+      A NetworkX graph
 
     alpha : float, optional
       Damping parameter for PageRank, default=0.85
-       
-    personalization: dict, optional      
+
+    personalization: dict, optional
        The "personalization vector" consisting of a dictionary with a
        key for every graph node and nonzero personalization value for each node.
 
@@ -40,9 +38,9 @@ def pagerank(G,alpha=0.85,personalization=None,
       Error tolerance used to check convergence in power method solver.
 
     nstart : dictionary, optional
-      Starting value of PageRank iteration for each node. 
+      Starting value of PageRank iteration for each node.
 
-    weight : key, optional       
+    weight : key, optional
       Edge data key to use as weight.  If None weights are set to 1.
 
     Returns
@@ -66,29 +64,32 @@ def pagerank(G,alpha=0.85,personalization=None,
     algorithm does not check if the input graph is directed and will
     execute on undirected graphs by converting each oriented edge in the
     directed graph to two edges.
-    
+
     See Also
     --------
     pagerank_numpy, pagerank_scipy, google_matrix
 
     References
     ----------
-    .. [1] A. Langville and C. Meyer, 
-       "A survey of eigenvector methods of web information retrieval."  
+    .. [1] A. Langville and C. Meyer,
+       "A survey of eigenvector methods of web information retrieval."
        http://citeseer.ist.psu.edu/713792.html
-    .. [2] Page, Lawrence; Brin, Sergey; Motwani, Rajeev and Winograd, Terry, 
+    .. [2] Page, Lawrence; Brin, Sergey; Motwani, Rajeev and Winograd, Terry,
        The PageRank citation ranking: Bringing order to the Web. 1999
        http://dbpubs.stanford.edu:8090/pub/showDoc.Fulltext?lang=en&doc=1999-66&format=pdf
     """
     if type(G) == nx.MultiGraph or type(G) == nx.MultiDiGraph:
         raise Exception("pagerank() not defined for graphs with multiedges.")
 
+    if len(G) == 0:
+        return {}
+
     if not G.is_directed():
         D=G.to_directed()
     else:
         D=G
 
-    # create a copy in (right) stochastic form        
+    # create a copy in (right) stochastic form
     W=nx.stochastic_graph(D, weight=weight)
     scale=1.0/W.number_of_nodes()
 
@@ -97,7 +98,7 @@ def pagerank(G,alpha=0.85,personalization=None,
         x=dict.fromkeys(W,scale)
     else:
         x=nstart
-        # normalize starting vector to 1                
+        # normalize starting vector to 1
         s=1.0/sum(x.values())
         for k in x: x[k]*=s
 
@@ -106,18 +107,18 @@ def pagerank(G,alpha=0.85,personalization=None,
         p=dict.fromkeys(W,scale)
     else:
         p=personalization
-        # normalize starting vector to 1                
+        # normalize starting vector to 1
         s=1.0/sum(p.values())
-        for k in p: 
+        for k in p:
             p[k]*=s
         if set(p)!=set(G):
-            raise NetworkXError('Personalization vector ' 
+            raise NetworkXError('Personalization vector '
                                 'must have a value for every node')
 
 
     # "dangling" nodes, no links out from them
     out_degree=W.out_degree()
-    dangle=[n for n in W if out_degree[n]==0.0]  
+    dangle=[n for n in W if out_degree[n]==0.0]
     i=0
     while True: # power iteration: make up to max_iter iterations
         xlast=x
@@ -129,17 +130,17 @@ def pagerank(G,alpha=0.85,personalization=None,
             for nbr in W[n]:
                 x[nbr]+=alpha*xlast[n]*W[n][nbr][weight]
             x[n]+=danglesum+(1.0-alpha)*p[n]
-        # normalize vector 
+        # normalize vector
         s=1.0/sum(x.values())
-        for n in x: 
+        for n in x:
             x[n]*=s
-        # check convergence, l1 norm            
+        # check convergence, l1 norm
         err=sum([abs(x[n]-xlast[n]) for n in x])
         if err < tol:
             break
         if i>max_iter:
-            raise NetworkXError('pagerank: power iteration failed to converge'
-                                'in %d iterations.'%(i+1))
+            raise NetworkXError('pagerank: power iteration failed to converge '
+                                'in %d iterations.'%(i-1))
         i+=1
     return x
 
@@ -151,20 +152,20 @@ def google_matrix(G, alpha=0.85, personalization=None,
     Parameters
     -----------
     G : graph
-      A NetworkX graph 
+      A NetworkX graph
 
     alpha : float
       The damping factor
 
-    personalization: dict, optional      
+    personalization: dict, optional
        The "personalization vector" consisting of a dictionary with a
        key for every graph node and nonzero personalization value for each node.
 
-    nodelist : list, optional       
+    nodelist : list, optional
       The rows and columns are ordered according to the nodes in nodelist.
       If nodelist is None, then the ordering is produced by G.nodes().
 
-    weight : key, optional       
+    weight : key, optional
       Edge data key to use as weight.  If None weights are set to 1.
 
     Returns
@@ -191,16 +192,18 @@ def google_matrix(G, alpha=0.85, personalization=None,
                                 'must have a value for every node')
     M=nx.to_numpy_matrix(G,nodelist=nodelist,weight=weight)
     (n,m)=M.shape # should be square
+    if n == 0:
+        return M
     # add constant to dangling nodes' row
     dangling=np.where(M.sum(axis=1)==0)
     for d in dangling[0]:
         M[d]=1.0/n
-    # normalize        
+    # normalize
     M=M/M.sum(axis=1)
     # add "teleportation"/personalization
     e=np.ones((n))
     if personalization is not None:
-        v=np.array(personalization.values()).astype(np.float)
+        v=np.array(list(personalization.values()),dtype=float)
     else:
         v=e
     v=v/v.sum()
@@ -214,20 +217,20 @@ def pagerank_numpy(G, alpha=0.85, personalization=None, weight='weight'):
     PageRank computes a ranking of the nodes in the graph G based on
     the structure of the incoming links. It was originally designed as
     an algorithm to rank web pages.
-    
+
     Parameters
     -----------
     G : graph
-      A NetworkX graph 
+      A NetworkX graph
 
     alpha : float, optional
       Damping parameter for PageRank, default=0.85
-       
-    personalization: dict, optional      
+
+    personalization: dict, optional
        The "personalization vector" consisting of a dictionary with a
        key for every graph node and nonzero personalization value for each node.
 
-    weight : key, optional       
+    weight : key, optional
       Edge data key to use as weight.  If None weights are set to 1.
 
     Returns
@@ -247,17 +250,17 @@ def pagerank_numpy(G, alpha=0.85, personalization=None, weight='weight'):
     for small graphs.
 
     This implementation works with Multi(Di)Graphs.
-    
+
     See Also
     --------
     pagerank, pagerank_scipy, google_matrix
 
     References
     ----------
-    .. [1] A. Langville and C. Meyer, 
-       "A survey of eigenvector methods of web information retrieval."  
+    .. [1] A. Langville and C. Meyer,
+       "A survey of eigenvector methods of web information retrieval."
        http://citeseer.ist.psu.edu/713792.html
-    .. [2] Page, Lawrence; Brin, Sergey; Motwani, Rajeev and Winograd, Terry, 
+    .. [2] Page, Lawrence; Brin, Sergey; Motwani, Rajeev and Winograd, Terry,
        The PageRank citation ranking: Bringing order to the Web. 1999
        http://dbpubs.stanford.edu:8090/pub/showDoc.Fulltext?lang=en&doc=1999-66&format=pdf
     """
@@ -265,7 +268,8 @@ def pagerank_numpy(G, alpha=0.85, personalization=None, weight='weight'):
         import numpy as np
     except ImportError:
         raise ImportError("pagerank_numpy() requires NumPy: http://scipy.org/")
-
+    if len(G) == 0:
+        return {}
     # choose ordering in matrix
     if personalization is None: # use G.nodes() ordering
         nodelist=G.nodes()
@@ -277,9 +281,9 @@ def pagerank_numpy(G, alpha=0.85, personalization=None, weight='weight'):
     eigenvalues,eigenvectors=np.linalg.eig(M.T)
     ind=eigenvalues.argsort()
     # eigenvector of largest eigenvalue at ind[-1], normalized
-    largest=np.array(eigenvectors[:,ind[-1]]).flatten().astype(np.float)
-    norm=largest.sum()
-    centrality=dict(zip(nodelist,largest/norm))
+    largest=np.array(eigenvectors[:,ind[-1]]).flatten().real
+    norm=float(largest.sum())
+    centrality=dict(zip(nodelist,map(float,largest/norm)))
     return centrality
 
 
@@ -290,16 +294,16 @@ def pagerank_scipy(G, alpha=0.85, personalization=None,
     PageRank computes a ranking of the nodes in the graph G based on
     the structure of the incoming links. It was originally designed as
     an algorithm to rank web pages.
-    
+
     Parameters
     -----------
     G : graph
-      A NetworkX graph 
+      A NetworkX graph
 
     alpha : float, optional
       Damping parameter for PageRank, default=0.85
-       
-    personalization: dict, optional      
+
+    personalization: dict, optional
        The "personalization vector" consisting of a dictionary with a
        key for every graph node and nonzero personalization value for each node.
 
@@ -309,7 +313,7 @@ def pagerank_scipy(G, alpha=0.85, personalization=None,
     tol : float, optional
       Error tolerance used to check convergence in power method solver.
 
-    weight : key, optional       
+    weight : key, optional
       Edge data key to use as weight.  If None weights are set to 1.
 
     Returns
@@ -326,17 +330,17 @@ def pagerank_scipy(G, alpha=0.85, personalization=None,
     -----
     The eigenvector calculation uses power iteration with a SciPy
     sparse matrix representation.
-    
+
     See Also
     --------
     pagerank, pagerank_numpy, google_matrix
 
     References
     ----------
-    .. [1] A. Langville and C. Meyer, 
-       "A survey of eigenvector methods of web information retrieval."  
+    .. [1] A. Langville and C. Meyer,
+       "A survey of eigenvector methods of web information retrieval."
        http://citeseer.ist.psu.edu/713792.html
-    .. [2] Page, Lawrence; Brin, Sergey; Motwani, Rajeev and Winograd, Terry, 
+    .. [2] Page, Lawrence; Brin, Sergey; Motwani, Rajeev and Winograd, Terry,
        The PageRank citation ranking: Bringing order to the Web. 1999
        http://dbpubs.stanford.edu:8090/pub/showDoc.Fulltext?lang=en&doc=1999-66&format=pdf
     """
@@ -344,21 +348,26 @@ def pagerank_scipy(G, alpha=0.85, personalization=None,
         import scipy.sparse
     except ImportError:
         raise ImportError("pagerank_scipy() requires SciPy: http://scipy.org/")
+    if len(G) == 0:
+        return {}
     # choose ordering in matrix
     if personalization is None: # use G.nodes() ordering
         nodelist=G.nodes()
     else:  # use personalization "vector" ordering
         nodelist=personalization.keys()
-    M=nx.to_scipy_sparse_matrix(G,nodelist=nodelist,weight=weight)
+    M=nx.to_scipy_sparse_matrix(G,nodelist=nodelist,weight=weight,dtype='f')
     (n,m)=M.shape # should be square
     S=scipy.array(M.sum(axis=1)).flatten()
-    for i, j, v in zip( *scipy.sparse.find(M) ):
-        M[i,j] = v / S[i]
+#    for i, j, v in zip( *scipy.sparse.find(M) ):
+#        M[i,j] = v / S[i]
+    S[S>0] = 1.0 / S[S>0]
+    Q = scipy.sparse.spdiags(S.T, 0, *M.shape, format='csr')
+    M = Q * M
     x=scipy.ones((n))/n  # initial guess
     dangle=scipy.array(scipy.where(M.sum(axis=1)==0,1.0/n,0)).flatten()
     # add "teleportation"/personalization
     if personalization is not None:
-        v=scipy.array(personalization.values()).astype(scipy.float_)
+        v=scipy.array(list(personalization.values()),dtype=float)
         v=v/v.sum()
     else:
         v=x
@@ -368,10 +377,10 @@ def pagerank_scipy(G, alpha=0.85, personalization=None,
         xlast=x
         x=alpha*(x*M+scipy.dot(dangle,xlast))+(1-alpha)*v
         x=x/x.sum()
-        # check convergence, l1 norm            
+        # check convergence, l1 norm
         err=scipy.absolute(x-xlast).sum()
         if err < n*tol:
-            return dict(zip(nodelist,x))
+            return dict(zip(nodelist,map(float,x)))
         i+=1
     raise NetworkXError('pagerank_scipy: power iteration failed to converge'
                         'in %d iterations.'%(i+1))
