@@ -2,25 +2,58 @@
 ********
 Matching
 ********
-
-The algorithm is taken from "Efficient Algorithms for Finding Maximum
-Matching in Graphs" by Zvi Galil, ACM Computing Surveys, 1986.
-It is based on the "blossom" method for finding augmenting paths and
-the "primal-dual" method for finding a matching of maximum weight, both
-methods invented by Jack Edmonds.
-
 """
-__author__ = """Joris van Rantwijk"""
-#    Copyright (C) 2004-2008 by 
+#    Copyright (C) 2004-2008 by
 #    Aric Hagberg <hagberg@lanl.gov>
 #    Dan Schult <dschult@colgate.edu>
 #    Pieter Swart <swart@lanl.gov>
 #    All rights reserved.
 #    BSD license.
-#
+#   Copyright (C) 2011 by
+#   Nicholas Mancuso <nick.mancuso@gmail.com>
+#   All rights reserved.
+#   BSD license.
 from itertools import repeat
+__author__ = """\n""".join(['Joris van Rantwijk',
+                            'Nicholas Mancuso (nick.mancuso@gmail.com)'])
+
+_all__ = ['max_weight_matching', 'maximal_matching']
+
+
+def maximal_matching(G):
+    r"""Find a maximal cardinality matching in the graph.
+
+    A matching is a subset of edges in which no node occurs more than once.
+    The cardinality of a matching is the number of matched edges.
+
+    Parameters
+    ----------
+    G : NetworkX graph
+        Undirected graph
+
+    Returns
+    -------
+    matching : set
+        A maximal matching of the graph.
+
+    Notes
+    -----
+    The algorithm greedily selects a maximal matching M of the graph G
+    (i.e. no superset of M exists). It runs in `O(|E|)` time.
+    """
+    matching = set([])
+    edges = set([])
+    for edge in G.edges_iter():
+        # If the edge isn't covered, add it to the matching
+        # then remove neighborhood of u and v from consideration.
+        if edge not in edges:
+            u, v = edge
+            matching.add(edge)
+            edges |= set(G.edges(u))
+            edges |= set(G.edges(v))
+
+    return matching
 
-_all__=['max_weight_matching']
 
 def max_weight_matching(G, maxcardinality=False):
     """Compute a maximum-weighted matching of G.
@@ -33,7 +66,7 @@ def max_weight_matching(G, maxcardinality=False):
     ----------
     G : NetworkX graph
       Undirected graph
-      
+
     maxcardinality: bool, optional
        If maxcardinality is True, compute the maximum-cardinality matching
        with maximum weight among all maximum-cardinality matchings.
@@ -44,7 +77,7 @@ def max_weight_matching(G, maxcardinality=False):
        The matching is returned as a dictionary, mate, such that
        mate[v] == w if node v is matched to node w.  Unmatched nodes do not
        occur as a key in mate.
-       
+
 
     Notes
     ------
@@ -58,12 +91,15 @@ def max_weight_matching(G, maxcardinality=False):
     could return a slightly suboptimal matching due to numeric
     precision errors.
 
+    This method is based on the "blossom" method for finding augmenting
+    paths and the "primal-dual" method for finding a matching of maximum
+    weight, both methods invented by Jack Edmonds [1]_.
+
     References
     ----------
-    .. [1] "Efficient Algorithms for Finding Maximum
-       Matching in Graphs" by Zvi Galil, ACM Computing Surveys, 1986.
+    .. [1] "Efficient Algorithms for Finding Maximum Matching in Graphs",
+       Zvi Galil, ACM Computing Surveys, 1986.
     """
-
     #
     # The algorithm is taken from "Efficient Algorithms for Finding Maximum
     # Matching in Graphs" by Zvi Galil, ACM Computing Surveys, 1986.
@@ -111,7 +147,7 @@ def max_weight_matching(G, maxcardinality=False):
     # Get a list of vertices.
     gnodes = G.nodes()
     if not gnodes:
-        return { } # don't bother with empty graphs
+        return { }  # don't bother with empty graphs
 
     # Find the maximum edge weight.
     maxweight = 0
@@ -120,7 +156,8 @@ def max_weight_matching(G, maxcardinality=False):
         wt=d.get('weight',1)
         if i != j and wt > maxweight:
             maxweight = wt
-        allinteger = allinteger and (type(wt) in (int, long))
+        allinteger = allinteger and (str(type(wt)).split("'")[1]
+                                     in ('int', 'long'))
 
     # If v is a matched vertex, mate[v] is its partner vertex.
     # If v is a single vertex, v does not occur as a key in mate.
@@ -339,7 +376,7 @@ def max_weight_matching(G, maxcardinality=False):
                     bestedgeto[bj] = k
             # Forget about least-slack edge of the subblossom.
             bestedge[bv] = None
-        b.mybestedges = bestedgeto.values()
+        b.mybestedges = list(bestedgeto.values())
         # Select bestedge[b].
         mybestedge = None
         bestedge[b] = None
@@ -588,7 +625,7 @@ def max_weight_matching(G, maxcardinality=False):
 
         # Make queue empty.
         queue[:] = [ ]
- 
+
         # Label single blossoms/vertices with S and put them in the queue.
         for v in gnodes:
             if (v not in mate) and label.get(inblossom[v]) is None:
@@ -744,7 +781,7 @@ def max_weight_matching(G, maxcardinality=False):
                         blossomdual[b] -= delta
 
             # Take action at the point where minimum delta occurred.
-            if deltatype == 1: 
+            if deltatype == 1:
                 # No further improvement possible; optimum reached.
                 break
             elif deltatype == 2:
@@ -774,7 +811,7 @@ def max_weight_matching(G, maxcardinality=False):
             break
 
         # End of a stage; expand all S-blossoms which have zero dual.
-        for b in blossomdual.keys():
+        for b in list(blossomdual.keys()):
             if b not in blossomdual:
                 continue # already expanded
             if ( blossomparent[b] is None and label.get(b) == 1 and
@@ -786,4 +823,3 @@ def max_weight_matching(G, maxcardinality=False):
         verifyOptimum()
 
     return mate
-
