@@ -1,30 +1,26 @@
 # -*- coding: utf-8 -*-
+"""Floyd-Warshall algorithm for shortest paths.
 """
-Floyd-Warshall algorithm for shortest paths.
-"""
-__author__ = """Aric Hagberg (hagberg@lanl.gov)"""
-#    Copyright (C) 2004-2011 by 
+#    Copyright (C) 2004-2012 by
 #    Aric Hagberg <hagberg@lanl.gov>
 #    Dan Schult <dschult@colgate.edu>
 #    Pieter Swart <swart@lanl.gov>
 #    All rights reserved.
 #    BSD license.
-
+import networkx as nx
+__author__ = """Aric Hagberg <aric.hagberg@gmail.com>"""
 __all__ = ['floyd_warshall',
            'floyd_warshall_predecessor_and_distance',
            'floyd_warshall_numpy']
 
-import networkx as nx
-
-
 def floyd_warshall_numpy(G, nodelist=None, weight='weight'):
     """Find all-pairs shortest path lengths using Floyd's algorithm.
 
     Parameters
     ----------
     G : NetworkX graph
-    
-    nodelist : list, optional       
+
+    nodelist : list, optional
        The rows and columns are ordered by the nodes in nodelist.
        If nodelist is None then the ordering is produced by G.nodes().
 
@@ -40,30 +36,29 @@ def floyd_warshall_numpy(G, nodelist=None, weight='weight'):
 
     Notes
     ------
-    Floyd's algorithm is appropriate for finding shortest paths 
-    in dense graphs or graphs with negative weights when Dijkstra's algorithm
-    fails.  This algorithm can still fail if there are negative cycles.
-    It has running time O(n^3) with running space is O(n^2).
+    Floyd's algorithm is appropriate for finding shortest paths in
+    dense graphs or graphs with negative weights when Dijkstra's
+    algorithm fails.  This algorithm can still fail if there are
+    negative cycles.  It has running time O(n^3) with running space of O(n^2).
     """
     try:
         import numpy as np
     except ImportError:
         raise ImportError(\
           "to_numpy_matrix() requires numpy: http://scipy.org/ ")
-    A=nx.to_numpy_matrix(G, nodelist=nodelist, multigraph_weight=min,
-                         weight=weight)
+    A = nx.to_numpy_matrix(G, nodelist=nodelist, multigraph_weight=min,
+                           weight=weight)
     n,m = A.shape
-    I=np.identity(n)
-    A[A==0]=np.inf # set zero entries to inf
-    A[I==1]=0 # except diagonal which should be zero
+    I = np.identity(n)
+    A[A==0] = np.inf # set zero entries to inf
+    A[I==1] = 0 # except diagonal which should be zero
     for i in range(n):
-        r = A[i,:]
-        A = np.minimum(A, r + r.T)
+        A = np.minimum(A, A[i,:] + A[:,i])
     return A
 
 def floyd_warshall_predecessor_and_distance(G, weight='weight'):
     """Find all-pairs shortest path lengths using Floyd's algorithm.
-    
+
     Parameters
     ----------
     G : NetworkX graph
@@ -74,15 +69,15 @@ def floyd_warshall_predecessor_and_distance(G, weight='weight'):
     Returns
     -------
     predecessor,distance : dictionaries
-       Dictionaries, keyed by source and target, of predecessors and distances 
+       Dictionaries, keyed by source and target, of predecessors and distances
        in the shortest path.
 
     Notes
     ------
-    Floyd's algorithm is appropriate for finding shortest paths 
+    Floyd's algorithm is appropriate for finding shortest paths
     in dense graphs or graphs with negative weights when Dijkstra's algorithm
     fails.  This algorithm can still fail if there are negative cycles.
-    It has running time O(n^3) with running space is O(n^2).
+    It has running time O(n^3) with running space of O(n^2).
 
     See Also
     --------
@@ -95,16 +90,17 @@ def floyd_warshall_predecessor_and_distance(G, weight='weight'):
     # dictionary-of-dictionaries representation for dist and pred
     # use some defaultdict magick here
     # for dist the default is the floating point inf value
-    dist=defaultdict(lambda : defaultdict(lambda: float('inf')))
-    pred=defaultdict(dict)
+    dist = defaultdict(lambda : defaultdict(lambda: float('inf')))
+    for u in G:
+        dist[u][u] = 0
+    pred = defaultdict(dict)
     # initialize path distance dictionary to be the adjacency matrix
     # also set the distance to self to 0 (zero diagonal)
-    undirected= not G.is_directed()
+    undirected = not G.is_directed()
     for u,v,d in G.edges(data=True):
         e_weight = d.get(weight, 1.0)
         dist[u][v] = min(e_weight, dist[u][v])
         pred[u][v] = u
-        dist[u][u] = 0
         if undirected:
             dist[v][u] = min(e_weight, dist[v][u])
             pred[v][u] = v
@@ -119,7 +115,7 @@ def floyd_warshall_predecessor_and_distance(G, weight='weight'):
 
 def floyd_warshall(G, weight='weight'):
     """Find all-pairs shortest path lengths using Floyd's algorithm.
-    
+
     Parameters
     ----------
     G : NetworkX graph
@@ -136,10 +132,10 @@ def floyd_warshall(G, weight='weight'):
 
     Notes
     ------
-    Floyd's algorithm is appropriate for finding shortest paths 
+    Floyd's algorithm is appropriate for finding shortest paths
     in dense graphs or graphs with negative weights when Dijkstra's algorithm
     fails.  This algorithm can still fail if there are negative cycles.
-    It has running time O(n^3) with running space is O(n^2).
+    It has running time O(n^3) with running space of O(n^2).
 
     See Also
     --------
@@ -148,7 +144,7 @@ def floyd_warshall(G, weight='weight'):
     all_pairs_shortest_path
     all_pairs_shortest_path_length
     """
-    # could make this it's own function to reduce memory costs
+    # could make this its own function to reduce memory costs
     return floyd_warshall_predecessor_and_distance(G, weight=weight)[1]
 
 # fixture for nose tests
