@@ -23,6 +23,9 @@ class TestGenericPath:
         assert_equal(nx.shortest_path(self.directed_cycle,0,3,weight='weight'),
                      [0, 1, 2, 3])
 
+    def test_shortest_path_target(self):
+        sp = nx.shortest_path(nx.path_graph(3), target=1)
+        assert_equal(sp, {0: [0, 1], 1: [1], 2: [2, 1]})
 
     def test_shortest_path_length(self):
         assert_equal(nx.shortest_path_length(self.cycle,0,3),3)
@@ -33,6 +36,11 @@ class TestGenericPath:
         assert_equal(nx.shortest_path_length(self.grid,1,12,weight='weight'),5)
         assert_equal(nx.shortest_path_length(self.directed_cycle,0,4,weight='weight'),4)
 
+    def test_shortest_path_length_target(self):
+        sp = nx.shortest_path_length(nx.path_graph(3), target=1)
+        assert_equal(sp[0], 1)
+        assert_equal(sp[1], 0)
+        assert_equal(sp[2], 1)
 
     def test_single_source_shortest_path(self):
         p=nx.shortest_path(self.cycle,0)
@@ -95,6 +103,7 @@ class TestGenericPath:
         l=nx.average_shortest_path_length(nx.path_graph(5))
         assert_almost_equal(l,2)
 
+
     def test_weighted_average_shortest_path(self):
         G=nx.Graph()
         G.add_cycle(range(7),weight=2)
@@ -111,7 +120,8 @@ class TestGenericPath:
         g.add_nodes_from(range(3))
         g.add_edge(0, 1)
         assert_raises(nx.NetworkXError,nx.average_shortest_path_length,g)
-
+        g = g.to_directed()
+        assert_raises(nx.NetworkXError,nx.average_shortest_path_length,g)
 
     def test_has_path(self):
         G = nx.Graph()
@@ -119,3 +129,17 @@ class TestGenericPath:
         G.add_path(range(3,5))
         assert_true(nx.has_path(G,0,2))
         assert_false(nx.has_path(G,0,4))
+
+    def test_all_shortest_paths(self):
+        G = nx.Graph()
+        G.add_path([0,1,2,3])
+        G.add_path([0,10,20,3])
+        assert_equal([[0,1,2,3],[0,10,20,3]],
+                     sorted(nx.all_shortest_paths(G,0,3)))
+
+    @raises(nx.NetworkXNoPath)
+    def test_all_shortest_paths_raise(self):
+        G = nx.Graph()
+        G.add_path([0,1,2,3])
+        G.add_node(4)
+        paths = list(nx.all_shortest_paths(G,0,4))
