@@ -35,10 +35,10 @@ class TestWeightedPath:
         self.MXG4=nx.MultiGraph(self.XG4)
         self.MXG4.add_edge(0,1,weight=3)
         self.G=nx.DiGraph()  # no weights
-        self.G.add_edges_from([('s','u'), ('s','x'), 
-                          ('u','v'), ('u','x'), 
-                          ('v','y'), ('x','u'), 
-                          ('x','v'), ('x','y'), 
+        self.G.add_edges_from([('s','u'), ('s','x'),
+                          ('u','v'), ('u','x'),
+                          ('v','y'), ('x','u'),
+                          ('x','v'), ('x','y'),
                           ('y','s'), ('y','v')])
 
     def test_dijkstra(self):
@@ -46,10 +46,10 @@ class TestWeightedPath:
         assert_equal(P['v'], ['s', 'x', 'u', 'v'])
         assert_equal(D['v'],9)
 
-        assert_equal(nx.single_source_dijkstra_path(self.XG,'s')['v'], 
+        assert_equal(nx.single_source_dijkstra_path(self.XG,'s')['v'],
                      ['s', 'x', 'u', 'v'])
         assert_equal(nx.single_source_dijkstra_path_length(self.XG,'s')['v'],9)
-        
+
         assert_equal(nx.single_source_dijkstra(self.XG,'s')[1]['v'],
                      ['s', 'x', 'u', 'v'])
 
@@ -57,6 +57,9 @@ class TestWeightedPath:
                      ['s', 'x', 'u', 'v'])
 
         GG=self.XG.to_undirected()
+        # make sure we get lower weight
+        # to_undirected might choose either edge with weight 2 or weight 3
+        GG['u']['x']['weight']=2
         (D,P)= nx.single_source_dijkstra(GG,'s')
         assert_equal(P['v'] , ['s', 'x', 'u', 'v'])
         assert_equal(D['v'],8)     # uses lower weight of 2 on u<->x edge
@@ -69,7 +72,6 @@ class TestWeightedPath:
         assert_equal(nx.dijkstra_path(self.XG4,0,2), [0, 1, 2])
         assert_equal(nx.dijkstra_path_length(self.XG4,0,2), 4)
         assert_equal(nx.dijkstra_path(self.MXG4,0,2), [0, 1, 2])
-
         assert_equal(nx.single_source_dijkstra(self.G,'s','v')[1]['v'],
                      ['s', 'u', 'v'])
         assert_equal(nx.single_source_dijkstra(self.G,'s')[1]['v'],
@@ -85,12 +87,16 @@ class TestWeightedPath:
         assert_equal(nx.dijkstra_path(self.cycle,0,3),[0, 1, 2, 3])
         assert_equal(nx.dijkstra_path(self.cycle,0,4), [0, 6, 5, 4])
 
+        assert_equal(nx.single_source_dijkstra(self.cycle,0,0),({0:0}, {0:[0]}) )
 
     def test_bidirectional_dijkstra(self):
         assert_equal(nx.bidirectional_dijkstra(self.XG, 's', 'v'),
                      (9, ['s', 'x', 'u', 'v']))
-        assert_equal(nx.bidirectional_dijkstra(self.G,'s','v'),
-                     (2, ['s', 'x', 'v']))
+        (dist,path) = nx.bidirectional_dijkstra(self.G,'s','v')
+        assert_equal(dist,2)
+        # skip this test, correct path could also be ['s','u','v']
+#        assert_equal(nx.bidirectional_dijkstra(self.G,'s','v'),
+#                     (2, ['s', 'x', 'v']))
         assert_equal(nx.bidirectional_dijkstra(self.cycle,0,3),
                      (3, [0, 1, 2, 3]))
         assert_equal(nx.bidirectional_dijkstra(self.cycle,0,4),
@@ -104,6 +110,14 @@ class TestWeightedPath:
         assert_equal(nx.dijkstra_path(self.XG,'s','v'),
                      nx.single_source_dijkstra_path(self.XG,'s')['v'])
 
+
+    @raises(nx.NetworkXNoPath)
+    def test_bidirectional_dijkstra_no_path(self):
+        G = nx.Graph()
+        G.add_path([1,2,3])
+        G.add_path([4,5,6])
+        path = nx.bidirectional_dijkstra(G,1,6)
+
     def test_dijkstra_predecessor(self):
         G=nx.path_graph(4)
         assert_equal(nx.dijkstra_predecessor_and_distance(G,0),
@@ -111,7 +125,7 @@ class TestWeightedPath:
         G=nx.grid_2d_graph(2,2)
         pred,dist=nx.dijkstra_predecessor_and_distance(G,(0,0))
         assert_equal(sorted(pred.items()),
-                     [((0, 0), []), ((0, 1), [(0, 0)]), 
+                     [((0, 0), []), ((0, 1), [(0, 0)]),
                       ((1, 0), [(0, 0)]), ((1, 1), [(0, 1), (1, 0)])])
         assert_equal(sorted(dist.items()),
                      [((0, 0), 0), ((0, 1), 1), ((1, 0), 1), ((1, 1), 2)])
@@ -125,19 +139,27 @@ class TestWeightedPath:
         (P,D)= nx.dijkstra_predecessor_and_distance(XG,'s')
         assert_equal(P['v'],['u'])
         assert_equal(D['v'],9)
+        (P,D)= nx.dijkstra_predecessor_and_distance(XG,'s',cutoff=8)
+        assert_false('v' in D)
+
+    def test_single_source_dijkstra_path_length(self):
+        pl = nx.single_source_dijkstra_path_length
+        assert_equal(pl(self.MXG4,0)[2], 4)
+        spl = pl(self.MXG4,0,cutoff=2)
+        assert_false(2 in spl)
 
     def test_bidirectional_dijkstra_multigraph(self):
-        G = nx.MultiGraph() 
-        G.add_edge('a', 'b', weight=10) 
-        G.add_edge('a', 'b', weight=100) 
+        G = nx.MultiGraph()
+        G.add_edge('a', 'b', weight=10)
+        G.add_edge('a', 'b', weight=100)
         dp= nx.bidirectional_dijkstra(G, 'a', 'b')
         assert_equal(dp,(10, ['a', 'b']))
 
 
     def test_dijkstra_pred_distance_multigraph(self):
-        G = nx.MultiGraph() 
-        G.add_edge('a', 'b', key='short',foo=5, weight=100) 
-        G.add_edge('a', 'b', key='long',bar=1, weight=110) 
+        G = nx.MultiGraph()
+        G.add_edge('a', 'b', key='short',foo=5, weight=100)
+        G.add_edge('a', 'b', key='long',bar=1, weight=110)
         p,d= nx.dijkstra_predecessor_and_distance(G, 'a')
         assert_equal(p,{'a': [], 'b': ['a']})
         assert_equal(d,{'a': 0, 'b': 100})
@@ -148,6 +170,11 @@ class TestWeightedPath:
         G.add_edge(8, 9, weight = -7)
         G.add_edge(9, 8, weight = 3)
         assert_equal(nx.negative_edge_cycle(G), True)
+        assert_raises(ValueError,nx.single_source_dijkstra_path_length,G,8)
+        assert_raises(ValueError,nx.single_source_dijkstra,G,8)
+        assert_raises(ValueError,nx.dijkstra_predecessor_and_distance,G,8)
+        G.add_edge(9,10)
+        assert_raises(ValueError,nx.bidirectional_dijkstra,G,8,10)
 
     def test_bellman_ford(self):
         # single node graph
@@ -167,7 +194,7 @@ class TestWeightedPath:
             assert_raises(nx.NetworkXUnbounded, nx.bellman_ford, G, i)
         # no negative cycle but negative weight
         G = nx.cycle_graph(5, create_using = nx.DiGraph())
-        G.add_edge(1, 2, weight = -3) 
+        G.add_edge(1, 2, weight = -3)
         assert_equal(nx.bellman_ford(G, 0),
                      ({0: None, 1: 0, 2: 1, 3: 2, 4: 3},
                       {0: 0, 1: 1, 2: -2, 3: -1, 4: 0}))
@@ -212,7 +239,7 @@ class TestWeightedPath:
         G=nx.grid_2d_graph(2,2)
         pred,dist=nx.bellman_ford(G,(0,0))
         assert_equal(sorted(pred.items()),
-                     [((0, 0), None), ((0, 1), (0, 0)), 
+                     [((0, 0), None), ((0, 1), (0, 0)),
                       ((1, 0), (0, 0)), ((1, 1), (0, 1))])
         assert_equal(sorted(dist.items()),
                      [((0, 0), 0), ((0, 1), 1), ((1, 0), 1), ((1, 1), 2)])
