@@ -120,8 +120,9 @@ class MultiDiGraph(MultiGraph,DiGraph):
     >>> G.node[1]
     {'time': '5pm'}
     >>> G.node[1]['room'] = 714
+    >>> del G.node[1]['room'] # remove attribute
     >>> G.nodes(data=True)
-    [(1, {'room': 714, 'time': '5pm'}), (3, {'time': '2pm'})]
+    [(1, {'time': '5pm'}), (3, {'time': '2pm'})]
 
     Warning: adding a node to G.node does not add it to the graph.
 
@@ -353,7 +354,7 @@ class MultiDiGraph(MultiGraph,DiGraph):
         Notes
         -----
         Nodes in nbunch that are not in the graph will be (quietly) ignored.
-        For directed graphs edges() is the same as out_edges().
+        For directed graphs this returns the out-edges.
 
         Examples
         --------
@@ -798,13 +799,16 @@ class MultiDiGraph(MultiGraph,DiGraph):
         bunch = self.nbunch_iter(nbunch)
         # create new graph and copy subgraph into it
         H = self.__class__()
+        # copy node and attribute dictionaries
+        for n in bunch:
+            H.node[n]=self.node[n]
         # namespace shortcuts for speed
         H_succ=H.succ
         H_pred=H.pred
         self_succ=self.succ
         self_pred=self.pred
         # add nodes
-        for n in bunch:
+        for n in H:
             H_succ[n]={}
             H_pred[n]={}
         # add edges
@@ -817,9 +821,6 @@ class MultiDiGraph(MultiGraph,DiGraph):
                     ed=edgedict.copy()
                     Hnbrs[v]=ed
                     H_pred[v][u]=ed
-        # copy node and attribute dictionaries
-        for n in H:
-            H.node[n]=self.node[n]
         H.graph=self.graph
         return H
 
