@@ -119,8 +119,9 @@ class MultiGraph(Graph):
     >>> G.node[1]
     {'time': '5pm'}
     >>> G.node[1]['room'] = 714
+    >>> del G.node[1]['room'] # remove attribute
     >>> G.nodes(data=True)
-    [(1, {'room': 714, 'time': '5pm'}), (3, {'time': '2pm'})]
+    [(1, {'time': '5pm'}), (3, {'time': '2pm'})]
 
     Warning: adding a node to G.node does not add it to the graph.
 
@@ -420,6 +421,7 @@ class MultiGraph(Graph):
 
                 - 2-tuples (u,v) All edges between u and v are removed.
                 - 3-tuples (u,v,key) The edge identified by key is removed.
+                - 4-tuples (u,v,key,data) where data is ignored.
 
         See Also
         --------
@@ -449,7 +451,7 @@ class MultiGraph(Graph):
         """
         for e in ebunch:
             try:
-                self.remove_edge(*e)
+                self.remove_edge(*e[:3])
             except NetworkXError:
                 pass
 
@@ -537,6 +539,7 @@ class MultiGraph(Graph):
         Notes
         -----
         Nodes in nbunch that are not in the graph will be (quietly) ignored.
+        For directed graphs this returns the out-edges.
 
         Examples
         --------
@@ -586,6 +589,7 @@ class MultiGraph(Graph):
         Notes
         -----
         Nodes in nbunch that are not in the graph will be (quietly) ignored.
+        For directed graphs this returns the out-edges.
 
         Examples
         --------
@@ -941,11 +945,14 @@ class MultiGraph(Graph):
         bunch =self.nbunch_iter(nbunch)
         # create new graph and copy subgraph into it
         H = self.__class__()
+        # copy node and attribute dictionaries
+        for n in bunch:
+            H.node[n]=self.node[n]
         # namespace shortcuts for speed
         H_adj=H.adj
         self_adj=self.adj
         # add nodes and edges (undirected method)
-        for n in bunch:
+        for n in H:
             Hnbrs={}
             H_adj[n]=Hnbrs
             for nbr,edgedict in self_adj[n].items():
@@ -955,8 +962,5 @@ class MultiGraph(Graph):
                     ed=edgedict.copy()
                     Hnbrs[nbr]=ed
                     H_adj[nbr][n]=ed
-        # copy node and attribute dictionaries
-        for n in H:
-            H.node[n]=self.node[n]
         H.graph=self.graph
         return H
