@@ -1,6 +1,8 @@
 #!/usr/bin/env python
+import random
 from nose.tools import *
 import networkx
+import networkx as nx
 
 class TestFunction(object):
     def setUp(self):
@@ -35,8 +37,8 @@ class TestFunction(object):
         assert_equal(self.DG.degree(),networkx.degree(self.DG))
         assert_equal(self.G.degree(nbunch=[0,1]),networkx.degree(self.G,nbunch=[0,1]))
         assert_equal(self.DG.degree(nbunch=[0,1]),networkx.degree(self.DG,nbunch=[0,1]))
-        assert_equal(self.G.degree(weighted=True),networkx.degree(self.G,weighted=True))
-        assert_equal(self.DG.degree(weighted=True),networkx.degree(self.DG,weighted=True))
+        assert_equal(self.G.degree(weight='weight'),networkx.degree(self.G,weight='weight'))
+        assert_equal(self.DG.degree(weight='weight'),networkx.degree(self.DG,weight='weight'))
     def test_neighbors(self):
         assert_equal(self.G.neighbors(1),networkx.neighbors(self.G,1))
         assert_equal(self.DG.neighbors(1),networkx.neighbors(self.DG,1))
@@ -67,9 +69,21 @@ class TestFunction(object):
 
     def test_degree_histogram(self):
         assert_equal(networkx.degree_histogram(self.G), [1,1,1,1,1])
+
     def test_density(self):
         assert_equal(networkx.density(self.G), 0.5)
         assert_equal(networkx.density(self.DG), 0.3)
+        G=networkx.Graph()
+        G.add_node(1)
+        assert_equal(networkx.density(G), 0.0)
+
+    def test_density_selfloop(self):
+        G = nx.Graph()
+        G.add_edge(1,1)
+        assert_equal(networkx.density(G), 0.0)
+        G.add_edge(1,2)
+        assert_equal(networkx.density(G), 2.0)
+
     def test_freeze(self):
         G=networkx.freeze(self.G)
         assert_equal(G.frozen,True)
@@ -124,3 +138,53 @@ class TestFunction(object):
              'Degree: 2',
              'Neighbors: 2'])
         assert_equal(info,expected_node_info)
+
+        assert_raises(networkx.NetworkXError,networkx.info,G,n=-1)
+
+    def test_neighbors(self):
+        graph = nx.complete_graph(100)
+        pop = random.sample(graph.nodes(), 1)
+        nbors = list(nx.neighbors(graph, pop[0]))
+        # should be all the other vertices in the graph
+        assert_equal(len(nbors), len(graph) - 1)
+
+        graph = nx.path_graph(100)
+        node = random.sample(graph.nodes(), 1)[0]
+        nbors = list(nx.neighbors(graph, node))
+        # should be all the other vertices in the graph
+        if node != 0 and node != 99:
+            assert_equal(len(nbors), 2)
+        else:
+            assert_equal(len(nbors), 1)
+
+        # create a star graph with 99 outer nodes
+        graph = nx.star_graph(99)
+        nbors = list(nx.neighbors(graph, 0))
+        assert_equal(len(nbors), 99)
+
+    def test_non_neighbors(self):
+        graph = nx.complete_graph(100)
+        pop = random.sample(graph.nodes(), 1)
+        nbors = list(nx.non_neighbors(graph, pop[0]))
+        # should be all the other vertices in the graph
+        assert_equal(len(nbors), 0)
+
+        graph = nx.path_graph(100)
+        node = random.sample(graph.nodes(), 1)[0]
+        nbors = list(nx.non_neighbors(graph, node))
+        # should be all the other vertices in the graph
+        if node != 0 and node != 99:
+            assert_equal(len(nbors), 97)
+        else:
+            assert_equal(len(nbors), 98)
+
+        # create a star graph with 99 outer nodes
+        graph = nx.star_graph(99)
+        nbors = list(nx.non_neighbors(graph, 0))
+        assert_equal(len(nbors), 0)
+
+        # disconnected graph
+        graph = nx.Graph()
+        graph.add_nodes_from(range(10))
+        nbors = list(nx.non_neighbors(graph, 0))
+        assert_equal(len(nbors), 9)
