@@ -3,7 +3,7 @@
 Matplotlib
 **********
 
-Draw networks with matplotlib (pylab).
+Draw networks with matplotlib.
 
 See Also
 --------
@@ -13,14 +13,16 @@ matplotlib:     http://matplotlib.sourceforge.net/
 pygraphviz:     http://networkx.lanl.gov/pygraphviz/
 
 """
-__author__ = """Aric Hagberg (hagberg@lanl.gov)"""
-#    Copyright (C) 2004-2010 by 
+#    Copyright (C) 2004-2012 by
 #    Aric Hagberg <hagberg@lanl.gov>
 #    Dan Schult <dschult@colgate.edu>
 #    Pieter Swart <swart@lanl.gov>
 #    All rights reserved.
 #    BSD license.
-
+import networkx as nx
+from networkx.drawing.layout import shell_layout,\
+    circular_layout,spectral_layout,spring_layout,random_layout
+__author__ = """Aric Hagberg (hagberg@lanl.gov)"""
 __all__ = ['draw',
            'draw_networkx',
            'draw_networkx_nodes',
@@ -34,37 +36,32 @@ __all__ = ['draw',
            'draw_shell',
            'draw_graphviz']
 
-
-import networkx as nx
-from networkx.drawing.layout import shell_layout,\
-    circular_layout,spectral_layout,spring_layout,random_layout
-
 def draw(G, pos=None, ax=None, hold=None, **kwds):
-    """Draw the graph G with Matplotlib (pylab).
+    """Draw the graph G with Matplotlib.
 
     Draw the graph as a simple representation with no node
     labels or edge labels and using the full Matplotlib figure area
     and no axis labels by default.  See draw_networkx() for more
     full-featured drawing that allows title, axis labels etc.
-    
+
     Parameters
     ----------
     G : graph
-       A networkx graph 
+       A networkx graph
 
     pos : dictionary, optional
        A dictionary with nodes as keys and positions as values.
        If not specified a spring layout positioning will be computed.
        See networkx.layout for functions that compute node positions.
-       
-    ax : Matplotlib Axes object, optional
-       Draw the graph in specified Matplotlib axes.  
 
-    hold: bool, optional
+    ax : Matplotlib Axes object, optional
+       Draw the graph in specified Matplotlib axes.
+
+    hold : bool, optional
        Set the Matplotlib hold state.  If True subsequent draw
        commands will be added to the current axes.
 
-    **kwds: optional keywords
+    **kwds : optional keywords
        See networkx.draw_networkx() for a description of optional keywords.
 
     Examples
@@ -90,17 +87,7 @@ def draw(G, pos=None, ax=None, hold=None, **kwds):
 
     since you might overwrite the pylab.draw function.
 
-    Good alternatives are:
-
-    With pylab:
-
-    >>> import pylab as P # 
-    >>> import networkx as nx
-    >>> G=nx.dodecahedral_graph()
-    >>> nx.draw(G)  # networkx draw()
-    >>> P.draw()    # pylab draw()
-    
-    With pyplot
+    With pyplot use
 
     >>> import matplotlib.pyplot as plt
     >>> import networkx as nx
@@ -110,18 +97,19 @@ def draw(G, pos=None, ax=None, hold=None, **kwds):
 
     Also see the NetworkX drawing examples at
     http://networkx.lanl.gov/gallery.html
-
-
     """
     try:
-        import matplotlib.pylab as pylab
+        import matplotlib.pyplot as plt
     except ImportError:
         raise ImportError("Matplotlib required for draw()")
     except RuntimeError:
         print("Matplotlib unable to open display")
         raise
 
-    cf=pylab.gcf()
+    if ax is None:
+        cf = plt.gcf()
+    else:
+        cf = ax.get_figure()
     cf.set_facecolor('w')
     if ax is None:
         if cf._axstack() is None:
@@ -130,18 +118,18 @@ def draw(G, pos=None, ax=None, hold=None, **kwds):
             ax=cf.gca()
 
  # allow callers to override the hold state by passing hold=True|False
-    b = pylab.ishold()
+    b = plt.ishold()
     h = kwds.pop('hold', None)
     if h is not None:
-        pylab.hold(h)
+        plt.hold(h)
     try:
         draw_networkx(G,pos=pos,ax=ax,**kwds)
         ax.set_axis_off()
-        pylab.draw_if_interactive()
+        plt.draw_if_interactive()
     except:
-        pylab.hold(b)
+        plt.hold(b)
         raise
-    pylab.hold(b)
+    plt.hold(b)
     return
 
 
@@ -151,85 +139,91 @@ def draw_networkx(G, pos=None, with_labels=True, **kwds):
     Draw the graph with Matplotlib with options for node positions,
     labeling, titles, and many other drawing features.
     See draw() for simple drawing without labels or axes.
-    
+
     Parameters
     ----------
     G : graph
-       A networkx graph 
+       A networkx graph
 
     pos : dictionary, optional
        A dictionary with nodes as keys and positions as values.
        If not specified a spring layout positioning will be computed.
        See networkx.layout for functions that compute node positions.
-       
+
+    with_labels :  bool, optional (default=True)
+       Set to True to draw labels on the nodes.
+
     ax : Matplotlib Axes object, optional
-       Draw the graph in the specified Matplotlib axes.  
+       Draw the graph in the specified Matplotlib axes.
 
-    with_labels:  bool, optional       
-       Set to True (default) to draw labels on the nodes.
+    nodelist : list, optional (default G.nodes())
+       Draw only specified nodes
 
-    nodelist: list, optional
-       Draw only specified nodes (default G.nodes())
+    edgelist : list, optional (default=G.edges())
+       Draw only specified edges
 
-    edgelist: list
-       Draw only specified edges(default=G.edges())
+    node_size : scalar or array, optional (default=300)
+       Size of nodes.  If an array is specified it must be the
+       same length as nodelist.
 
-    node_size: scalar or array
-       Size of nodes (default=300).  If an array is specified it must be the
-       same length as nodelist. 
-
-    node_color: color string, or array of floats
-       Node color. Can be a single color format string (default='r'),
+    node_color : color string, or array of floats, (default='r')
+       Node color. Can be a single color format string,
        or a  sequence of colors with the same length as nodelist.
        If numeric values are specified they will be mapped to
        colors using the cmap and vmin,vmax parameters.  See
        matplotlib.scatter for more details.
 
-    node_shape:  string
+    node_shape :  string, optional (default='o')
        The shape of the node.  Specification is as matplotlib.scatter
-       marker, one of 'so^>v<dph8' (default='o').
+       marker, one of 'so^>v<dph8'.
 
-    alpha: float
-       The node transparency (default=1.0) 
+    alpha : float, optional (default=1.0)
+       The node transparency
 
-    cmap: Matplotlib colormap
-       Colormap for mapping intensities of nodes (default=None)
+    cmap : Matplotlib colormap, optional (default=None)
+       Colormap for mapping intensities of nodes
 
-    vmin,vmax: floats
-       Minimum and maximum for node colormap scaling (default=None)
+    vmin,vmax : float, optional (default=None)
+       Minimum and maximum for node colormap scaling
 
-    width`: float
-       Line width of edges (default =1.0)
+    linewidths : [None | scalar | sequence]
+       Line width of symbol border (default =1.0)
 
-    edge_color: color string, or array of floats
-       Edge color. Can be a single color format string (default='r'),
+    width : float, optional (default=1.0)
+       Line width of edges
+
+    edge_color : color string, or array of floats (default='r')
+       Edge color. Can be a single color format string,
        or a sequence of colors with the same length as edgelist.
        If numeric values are specified they will be mapped to
        colors using the edge_cmap and edge_vmin,edge_vmax parameters.
 
-    edge_ cmap: Matplotlib colormap
-       Colormap for mapping intensities of edges (default=None)
+    edge_ cmap : Matplotlib colormap, optional (default=None)
+       Colormap for mapping intensities of edges
 
-    edge_vmin,edge_vmax: floats
-       Minimum and maximum for edge colormap scaling (default=None)
+    edge_vmin,edge_vmax : floats, optional (default=None)
+       Minimum and maximum for edge colormap scaling
 
-    style: string
-       Edge line style (default='solid') (solid|dashed|dotted,dashdot)
+    style : string, optional (default='solid')
+       Edge line style (solid|dashed|dotted,dashdot)
 
-    labels: dictionary
-       Node labels in a dictionary keyed by node of text labels (default=None)
+    labels : dictionary, optional (default=None)
+       Node labels in a dictionary keyed by node of text labels
 
-    font_size: int
-       Font size for text labels (default=12)
+    font_size : int, optional (default=12)
+       Font size for text labels
 
-    font_color: string
-       Font color string (default='k' black)
+    font_color : string, optional (default='k' black)
+       Font color string
 
-    font_weight: string
-       Font weight (default='normal')
+    font_weight : string, optional (default='normal')
+       Font weight
 
-    font_family: string
-       Font family (default='sans-serif')
+    font_family : string, optional (default='sans-serif')
+       Font family
+
+    label : string, optional
+        Label for graph legend
 
     Examples
     --------
@@ -237,8 +231,8 @@ def draw_networkx(G, pos=None, with_labels=True, **kwds):
     >>> nx.draw(G)
     >>> nx.draw(G,pos=nx.spring_layout(G)) # use spring layout
 
-    >>> import pylab
-    >>> limits=pylab.axis('off') # turn of axis 
+    >>> import matplotlib.pyplot as plt
+    >>> limits=plt.axis('off') # turn of axis
 
     Also see the NetworkX drawing examples at
     http://networkx.lanl.gov/gallery.html
@@ -250,10 +244,9 @@ def draw_networkx(G, pos=None, with_labels=True, **kwds):
     draw_networkx_edges()
     draw_networkx_labels()
     draw_networkx_edge_labels()
-
     """
     try:
-        import matplotlib.pylab as pylab
+        import matplotlib.pyplot as plt
     except ImportError:
         raise ImportError("Matplotlib required for draw()")
     except RuntimeError:
@@ -264,10 +257,10 @@ def draw_networkx(G, pos=None, with_labels=True, **kwds):
         pos=nx.drawing.spring_layout(G) # default to spring layout
 
     node_collection=draw_networkx_nodes(G, pos, **kwds)
-    edge_collection=draw_networkx_edges(G, pos, **kwds) 
+    edge_collection=draw_networkx_edges(G, pos, **kwds)
     if with_labels:
         draw_networkx_labels(G, pos, **kwds)
-    pylab.draw_if_interactive()
+    plt.draw_if_interactive()
 
 def draw_networkx_nodes(G, pos,
                         nodelist=None,
@@ -277,9 +270,10 @@ def draw_networkx_nodes(G, pos,
                         alpha=1.0,
                         cmap=None,
                         vmin=None,
-                        vmax=None, 
+                        vmax=None,
                         ax=None,
                         linewidths=None,
+                        label = None,
                         **kwds):
     """Draw the nodes of the graph G.
 
@@ -288,54 +282,53 @@ def draw_networkx_nodes(G, pos,
     Parameters
     ----------
     G : graph
-       A networkx graph 
+       A networkx graph
 
     pos : dictionary
        A dictionary with nodes as keys and positions as values.
        If not specified a spring layout positioning will be computed.
        See networkx.layout for functions that compute node positions.
-       
-    ax : Matplotlib Axes object, optional
-       Draw the graph in the specified Matplotlib axes.  
 
-    nodelist: list, optional
+    ax : Matplotlib Axes object, optional
+       Draw the graph in the specified Matplotlib axes.
+
+    nodelist : list, optional
        Draw only specified nodes (default G.nodes())
 
-    edgelist: list
-       Draw only specified edges(default=G.edges())
-
-    node_size: scalar or array
+    node_size : scalar or array
        Size of nodes (default=300).  If an array is specified it must be the
-       same length as nodelist. 
+       same length as nodelist.
 
-    node_color: color string, or array of floats
+    node_color : color string, or array of floats
        Node color. Can be a single color format string (default='r'),
        or a  sequence of colors with the same length as nodelist.
        If numeric values are specified they will be mapped to
        colors using the cmap and vmin,vmax parameters.  See
        matplotlib.scatter for more details.
 
-    node_shape:  string
+    node_shape :  string
        The shape of the node.  Specification is as matplotlib.scatter
        marker, one of 'so^>v<dph8' (default='o').
 
-    alpha: float
-       The node transparency (default=1.0) 
+    alpha : float
+       The node transparency (default=1.0)
 
-    cmap: Matplotlib colormap
+    cmap : Matplotlib colormap
        Colormap for mapping intensities of nodes (default=None)
 
-    vmin,vmax: floats
+    vmin,vmax : floats
        Minimum and maximum for node colormap scaling (default=None)
 
-    width`: float
-       Line width of edges (default =1.0)
+    linewidths : [None | scalar | sequence]
+       Line width of symbol border (default =1.0)
 
+    label : [None| string]
+       Label for legend
 
     Examples
     --------
     >>> G=nx.dodecahedral_graph()
-    >>> nodes=nx.draw_networkx_nodes(G,pos=nx.spring_layout(G)) 
+    >>> nodes=nx.draw_networkx_nodes(G,pos=nx.spring_layout(G))
 
     Also see the NetworkX drawing examples at
     http://networkx.lanl.gov/gallery.html
@@ -347,12 +340,9 @@ def draw_networkx_nodes(G, pos,
     draw_networkx_edges()
     draw_networkx_labels()
     draw_networkx_edge_labels()
-
-
-
     """
     try:
-        import matplotlib.pylab as pylab
+        import matplotlib.pyplot as plt
         import numpy
     except ImportError:
         raise ImportError("Matplotlib required for draw()")
@@ -362,13 +352,13 @@ def draw_networkx_nodes(G, pos,
 
 
     if ax is None:
-        ax=pylab.gca()
+        ax=plt.gca()
 
     if nodelist is None:
         nodelist=G.nodes()
 
     if not nodelist or len(nodelist)==0:  # empty nodelist, no drawing
-        return None 
+        return None
 
     try:
         xy=numpy.asarray([pos[v] for v in nodelist])
@@ -383,15 +373,14 @@ def draw_networkx_nodes(G, pos,
                                s=node_size,
                                c=node_color,
                                marker=node_shape,
-                               cmap=cmap, 
+                               cmap=cmap,
                                vmin=vmin,
                                vmax=vmax,
                                alpha=alpha,
-                               linewidths=linewidths)
-                               
-#    pylab.axes(ax)
-    pylab.sci(node_collection)
-    node_collection.set_zorder(2)            
+                               linewidths=linewidths,
+                               label=label)
+
+    node_collection.set_zorder(2)
     return node_collection
 
 
@@ -403,9 +392,10 @@ def draw_networkx_edges(G, pos,
                         alpha=None,
                         edge_cmap=None,
                         edge_vmin=None,
-                        edge_vmax=None, 
+                        edge_vmax=None,
                         ax=None,
                         arrows=True,
+                        label=None,
                         **kwds):
     """Draw the edges of the graph G.
 
@@ -414,36 +404,45 @@ def draw_networkx_edges(G, pos,
     Parameters
     ----------
     G : graph
-       A networkx graph 
+       A networkx graph
 
     pos : dictionary
        A dictionary with nodes as keys and positions as values.
        If not specified a spring layout positioning will be computed.
        See networkx.layout for functions that compute node positions.
-       
-    ax : Matplotlib Axes object, optional
-       Draw the graph in the specified Matplotlib axes.  
 
-    alpha: float
-       The edge transparency (default=1.0) 
+    edgelist : collection of edge tuples
+       Draw only specified edges(default=G.edges())
 
-    width`: float
+    width : float
        Line width of edges (default =1.0)
 
-    edge_color: color string, or array of floats
+    edge_color : color string, or array of floats
        Edge color. Can be a single color format string (default='r'),
        or a sequence of colors with the same length as edgelist.
        If numeric values are specified they will be mapped to
        colors using the edge_cmap and edge_vmin,edge_vmax parameters.
 
-    edge_ cmap: Matplotlib colormap
+    style : string
+       Edge line style (default='solid') (solid|dashed|dotted,dashdot)
+
+    alpha : float
+       The edge transparency (default=1.0)
+
+    edge_ cmap : Matplotlib colormap
        Colormap for mapping intensities of edges (default=None)
 
-    edge_vmin,edge_vmax: floats
+    edge_vmin,edge_vmax : floats
        Minimum and maximum for edge colormap scaling (default=None)
 
-    style: string
-       Edge line style (default='solid') (solid|dashed|dotted,dashdot)
+    ax : Matplotlib Axes object, optional
+       Draw the graph in the specified Matplotlib axes.
+
+    arrows : bool, optional (default=True)
+       For directed graphs, if True draw arrowheads.
+
+    label : [None| string]
+       Label for legend
 
     Notes
     -----
@@ -455,7 +454,7 @@ def draw_networkx_edges(G, pos,
     Examples
     --------
     >>> G=nx.dodecahedral_graph()
-    >>> edges=nx.draw_networkx_edges(G,pos=nx.spring_layout(G)) 
+    >>> edges=nx.draw_networkx_edges(G,pos=nx.spring_layout(G))
 
     Also see the NetworkX drawing examples at
     http://networkx.lanl.gov/gallery.html
@@ -467,11 +466,10 @@ def draw_networkx_edges(G, pos,
     draw_networkx_nodes()
     draw_networkx_labels()
     draw_networkx_edge_labels()
-
     """
     try:
         import matplotlib
-        import matplotlib.pylab as pylab
+        import matplotlib.pyplot as plt
         import matplotlib.cbook as cb
         from matplotlib.colors import colorConverter,Colormap
         from matplotlib.collections import LineCollection
@@ -483,7 +481,7 @@ def draw_networkx_edges(G, pos,
         raise
 
     if ax is None:
-        ax=pylab.gca()
+        ax=plt.gca()
 
     if edgelist is None:
         edgelist=G.edges()
@@ -493,7 +491,7 @@ def draw_networkx_edges(G, pos,
 
     # set edge positions
     edge_pos=numpy.asarray([(pos[e[0]],pos[e[1]]) for e in edgelist])
-    
+
     if not cb.iterable(width):
         lw = (width,)
     else:
@@ -502,13 +500,13 @@ def draw_networkx_edges(G, pos,
     if not cb.is_string_like(edge_color) \
            and cb.iterable(edge_color) \
            and len(edge_color)==len(edge_pos):
-        if numpy.alltrue([cb.is_string_like(c) 
+        if numpy.alltrue([cb.is_string_like(c)
                          for c in edge_color]):
             # (should check ALL elements)
             # list of color letters such as ['k','r','k',...]
-            edge_colors = tuple([colorConverter.to_rgba(c,alpha) 
+            edge_colors = tuple([colorConverter.to_rgba(c,alpha)
                                  for c in edge_color])
-        elif numpy.alltrue([not cb.is_string_like(c) 
+        elif numpy.alltrue([not cb.is_string_like(c)
                            for c in edge_color]):
             # If color specs are given as (rgb) or (rgba) tuples, we're OK
             if numpy.alltrue([cb.iterable(c) and len(c) in (3,4)
@@ -529,12 +527,13 @@ def draw_networkx_edges(G, pos,
                                      colors       = edge_colors,
                                      linewidths   = lw,
                                      antialiaseds = (1,),
-                                     linestyle    = style,     
-                                     transOffset = ax.transData,             
+                                     linestyle    = style,
+                                     transOffset = ax.transData,
                                      )
 
 
-    edge_collection.set_zorder(1) # edges go behind nodes            
+    edge_collection.set_zorder(1) # edges go behind nodes
+    edge_collection.set_label(label)
     ax.add_collection(edge_collection)
 
     # Note: there was a bug in mpl regarding the handling of alpha values for
@@ -545,33 +544,15 @@ def draw_networkx_edges(G, pos,
     if cb.is_numlike(alpha):
         edge_collection.set_alpha(alpha)
 
-    # need 0.87.7 or greater for edge colormaps
-    mpl_version=matplotlib.__version__
-    if mpl_version.endswith('.svn'):
-        mpl_version=matplotlib.__version__[0:-4]
-    elif mpl_version.endswith('svn'):
-        mpl_version=matplotlib.__version__[0:-3]
-    elif mpl_version.endswith('pre'):
-        mpl_version=matplotlib.__version__[0:-3]
-    if list(map(int,mpl_version.split('.')))>=[0,87,7]:
-        if edge_colors is None:
-            if edge_cmap is not None: assert(isinstance(edge_cmap, Colormap))
-            edge_collection.set_array(numpy.asarray(edge_color))
-            edge_collection.set_cmap(edge_cmap)
-            if edge_vmin is not None or edge_vmax is not None:
-                edge_collection.set_clim(edge_vmin, edge_vmax)
-            else:
-                edge_collection.autoscale()
-#            pylab.axes(ax)
-            pylab.sci(edge_collection)
-
-#    else:
-#        sys.stderr.write(\
-#            """matplotlib version >= 0.87.7 required for colormapped edges.
-#        (version %s detected)."""%matplotlib.__version__)
-#        raise UserWarning(\
-#            """matplotlib version >= 0.87.7 required for colormapped edges.
-#        (version %s detected)."""%matplotlib.__version__)
+    if edge_colors is None:
+        if edge_cmap is not None:
+            assert(isinstance(edge_cmap, Colormap))
+        edge_collection.set_array(numpy.asarray(edge_color))
+        edge_collection.set_cmap(edge_cmap)
+        if edge_vmin is not None or edge_vmax is not None:
+            edge_collection.set_clim(edge_vmin, edge_vmax)
+        else:
+            edge_collection.autoscale()
 
     arrow_collection=None
 
@@ -579,7 +560,7 @@ def draw_networkx_edges(G, pos,
 
         # a directed graph hack
         # draw thick line segments at head end of edge
-        # waiting for someone else to implement arrows that will work 
+        # waiting for someone else to implement arrows that will work
         arrow_colors = edge_colors
         a_pos=[]
         p=1.0-0.25 # make head segment 25 percent of edge length
@@ -601,21 +582,22 @@ def draw_networkx_edges(G, pos,
                 theta=numpy.arctan2(dy,dx)
                 xa=p*d*numpy.cos(theta)+x1
                 ya=p*d*numpy.sin(theta)+y1
-                
+
             a_pos.append(((xa,ya),(x2,y2)))
 
         arrow_collection = LineCollection(a_pos,
                                 colors       = arrow_colors,
                                 linewidths   = [4*ww for ww in lw],
                                 antialiaseds = (1,),
-                                transOffset = ax.transData,             
+                                transOffset = ax.transData,
                                 )
-        
-        arrow_collection.set_zorder(1) # edges go behind nodes            
+
+        arrow_collection.set_zorder(1) # edges go behind nodes
+        arrow_collection.set_label(label)
         ax.add_collection(arrow_collection)
 
 
-    # update view        
+    # update view
     minx = numpy.amin(numpy.ravel(edge_pos[:,:,0]))
     maxx = numpy.amax(numpy.ravel(edge_pos[:,:,0]))
     miny = numpy.amin(numpy.ravel(edge_pos[:,:,1]))
@@ -647,33 +629,33 @@ def draw_networkx_labels(G, pos,
     Parameters
     ----------
     G : graph
-       A networkx graph 
+       A networkx graph
 
     pos : dictionary, optional
        A dictionary with nodes as keys and positions as values.
        If not specified a spring layout positioning will be computed.
        See networkx.layout for functions that compute node positions.
-       
-    ax : Matplotlib Axes object, optional
-       Draw the graph in the specified Matplotlib axes.  
 
-    alpha: float
-       The text transparency (default=1.0) 
+    labels : dictionary, optional (default=None)
+       Node labels in a dictionary keyed by node of text labels
 
-    labels: dictionary
-       Node labels in a dictionary keyed by node of text labels (default=None)
-
-    font_size: int
+    font_size : int
        Font size for text labels (default=12)
 
-    font_color: string
+    font_color : string
        Font color string (default='k' black)
 
-    font_weight: string
+    font_family : string
+       Font family (default='sans-serif')
+
+    font_weight : string
        Font weight (default='normal')
 
-    font_family: string
-       Font family (default='sans-serif')
+    alpha : float
+       The text transparency (default=1.0)
+
+    ax : Matplotlib Axes object, optional
+       Draw the graph in the specified Matplotlib axes.
 
 
     Examples
@@ -694,7 +676,7 @@ def draw_networkx_labels(G, pos,
     draw_networkx_edge_labels()
     """
     try:
-        import matplotlib.pylab as pylab
+        import matplotlib.pyplot as plt
         import matplotlib.cbook as cb
     except ImportError:
         raise ImportError("Matplotlib required for draw()")
@@ -703,16 +685,16 @@ def draw_networkx_labels(G, pos,
         raise
 
     if ax is None:
-        ax=pylab.gca()
+        ax=plt.gca()
 
     if labels is None:
-        labels=dict(list(zip(G.nodes(),G.nodes())))
+        labels=dict( (n,n) for n in G.nodes())
 
     # set optional alignment
     horizontalalignment=kwds.get('horizontalalignment','center')
     verticalalignment=kwds.get('verticalalignment','center')
 
-    text_items={}  # there is no text collection so we'll fake one        
+    text_items={}  # there is no text collection so we'll fake one
     for n, label in labels.items():
         (x,y)=pos[n]
         if not cb.is_string_like(label):
@@ -734,6 +716,7 @@ def draw_networkx_labels(G, pos,
 
 def draw_networkx_edge_labels(G, pos,
                               edge_labels=None,
+                              label_pos=0.5,
                               font_size=10,
                               font_color='k',
                               font_family='sans-serif',
@@ -748,46 +731,49 @@ def draw_networkx_edge_labels(G, pos,
     Parameters
     ----------
     G : graph
-       A networkx graph 
+       A networkx graph
 
     pos : dictionary, optional
        A dictionary with nodes as keys and positions as values.
        If not specified a spring layout positioning will be computed.
        See networkx.layout for functions that compute node positions.
-       
+
     ax : Matplotlib Axes object, optional
-       Draw the graph in the specified Matplotlib axes.  
+       Draw the graph in the specified Matplotlib axes.
 
-    alpha: float
-       The text transparency (default=1.0) 
+    alpha : float
+       The text transparency (default=1.0)
 
-    labels: dictionary
-       Node labels in a dictionary keyed by edge two-tuple of text
-       labels (default=None), Only labels for the keys in the dictionary
-       are drawn. 
+    edge_labels : dictionary
+       Edge labels in a dictionary keyed by edge two-tuple of text
+       labels (default=None). Only labels for the keys in the dictionary
+       are drawn.
 
-    font_size: int
+    label_pos : float
+       Position of edge label along edge (0=head, 0.5=center, 1=tail)
+
+    font_size : int
        Font size for text labels (default=12)
 
-    font_color: string
+    font_color : string
        Font color string (default='k' black)
 
-    font_weight: string
+    font_weight : string
        Font weight (default='normal')
 
-    font_family: string
+    font_family : string
        Font family (default='sans-serif')
 
-    bbox: Matplotlib bbox
+    bbox : Matplotlib bbox
        Specify text box shape and colors.
 
-    clip_on: bool
+    clip_on : bool
        Turn on clipping at axis boundaries (default=True)
 
     Examples
     --------
     >>> G=nx.dodecahedral_graph()
-    >>> edge_labels=nx.draw_networkx_edge_labels(G,pos=nx.spring_layout(G)) 
+    >>> edge_labels=nx.draw_networkx_edge_labels(G,pos=nx.spring_layout(G))
 
     Also see the NetworkX drawing examples at
     http://networkx.lanl.gov/gallery.html
@@ -799,10 +785,9 @@ def draw_networkx_edge_labels(G, pos,
     draw_networkx_nodes()
     draw_networkx_edges()
     draw_networkx_labels()
-
     """
     try:
-        import matplotlib.pylab as pylab
+        import matplotlib.pyplot as plt
         import matplotlib.cbook as cb
         import numpy
     except ImportError:
@@ -812,22 +797,24 @@ def draw_networkx_edge_labels(G, pos,
         raise
 
     if ax is None:
-        ax=pylab.gca()
+        ax=plt.gca()
     if edge_labels is None:
-        labels=dict(list(zip(G.edges(),[d for u,v,d in G.edges(data=True)])))
+        labels=dict( ((u,v), d) for u,v,d in G.edges(data=True) )
     else:
         labels = edge_labels
-    text_items={} 
+    text_items={}
     for (n1,n2), label in labels.items():
         (x1,y1)=pos[n1]
         (x2,y2)=pos[n2]
-        (x,y) = (float(x1+x2)/2.0, float(y1+y2)/2.0)
-        if rotate: 
+        (x,y) = (x1 * label_pos + x2 * (1.0 - label_pos),
+                 y1 * label_pos + y2 * (1.0 - label_pos))
+
+        if rotate:
             angle=numpy.arctan2(y2-y1,x2-x1)/(2.0*numpy.pi)*360 # degrees
             # make label orientation "right-side-up"
-            if angle > 90: 
+            if angle > 90:
                 angle-=180
-            if angle < - 90: 
+            if angle < - 90:
                 angle+=180
             # transform data coordinate angle to screen coordinate angle
             xy=numpy.array((x,y))
@@ -866,12 +853,10 @@ def draw_networkx_edge_labels(G, pos,
 
     return text_items
 
-
-
 def draw_circular(G, **kwargs):
     """Draw the graph G with a circular layout."""
     draw(G,circular_layout(G),**kwargs)
-    
+
 def draw_random(G, **kwargs):
     """Draw the graph G with a random layout."""
     draw(G,random_layout(G),**kwargs)
@@ -887,7 +872,7 @@ def draw_spring(G, **kwargs):
 def draw_shell(G, **kwargs):
     """Draw networkx graph with shell layout."""
     nlist = kwargs.get('nlist', None)
-    if nlist != None:        
+    if nlist != None:
         del(kwargs['nlist'])
     draw(G,shell_layout(G,nlist=nlist),**kwargs)
 
@@ -906,6 +891,6 @@ def setup_module(module):
     try:
         import matplotlib as mpl
         mpl.use('PS',warn=False)
-        import pylab
+        import matplotlib.pyplot as plt
     except:
         raise SkipTest("matplotlib not available")
