@@ -13,7 +13,7 @@ __author__ = """Aric Hagberg (hagberg@lanl.gov)\nDan Schult (dschult@colgate.edu
 _all__ = ['kl_connected_subgraph', 'is_kl_connected']
 
 import copy
-from networkx import shortest_path
+import networkx as nx
 
 def kl_connected_subgraph(G,k,l,low_memory=False,same_as_graph=False):
     """ Returns the maximum locally (k,l) connected subgraph of G.
@@ -59,7 +59,10 @@ def kl_connected_subgraph(G,k,l,low_memory=False,same_as_graph=False):
                         G2.remove_edge(prev,w)
                         prev=w
 #                path=shortest_path(G2,u,v,k) # ??? should "Cutoff" be k+1?
-                path=shortest_path(G2,u,v) # ??? should "Cutoff" be k+1?
+                try:
+                    path=nx.shortest_path(G2,u,v) # ??? should "Cutoff" be k+1?
+                except nx.NetworkXNoPath:
+                    path = False
             # No Other Paths
             if accept==0:
                 H.remove_edge(u,v)
@@ -72,7 +75,7 @@ def kl_connected_subgraph(G,k,l,low_memory=False,same_as_graph=False):
     return H
 
 def is_kl_connected(G,k,l,low_memory=False):
-    """Returns True if G is kl connected """
+    """Returns True if G is kl connected."""
     graphOK=True
     for edge in G.edges():
         (u,v)=edge
@@ -100,7 +103,10 @@ def is_kl_connected(G,k,l,low_memory=False):
                     G2.remove_edge(prev,w)
                     prev=w
 #            path=shortest_path(G2,u,v,k) # ??? should "Cutoff" be k+1?
-            path=shortest_path(G2,u,v) # ??? should "Cutoff" be k+1?
+            try:
+                path=nx.shortest_path(G2,u,v) # ??? should "Cutoff" be k+1?
+            except nx.NetworkXNoPath:
+                path = False
         # No Other Paths
         if accept==0:
             graphOK=False
