@@ -1,14 +1,8 @@
 #!/usr/bin/env python
-
 from nose.tools import *
 from networkx import *
 from networkx.generators.random_graphs import *
 
-
-"""Generators - Random Graphs
-----------------------------
-"""
-
 class TestGeneratorsRandom():
     def smoke_test_random_graph(self):
         seed = 42
@@ -26,7 +20,7 @@ class TestGeneratorsRandom():
         G=connected_watts_strogatz_graph(10,2,0.1,seed)
         assert_equal(len(G), 10)
         assert_equal(G.number_of_edges(), 10)
-        
+
         G=watts_strogatz_graph(10,4,0.25,seed)
         assert_equal(len(G), 10)
         assert_equal(G.number_of_edges(), 20)
@@ -42,13 +36,13 @@ class TestGeneratorsRandom():
         G=barabasi_albert_graph(100,1,seed)
         G=barabasi_albert_graph(100,3,seed)
         assert_equal(G.number_of_edges(),(97*3))
-        
+
         G=powerlaw_cluster_graph(100,1,1.0,seed)
         G=powerlaw_cluster_graph(100,3,0.0,seed)
         assert_equal(G.number_of_edges(),(97*3))
-        
+
         G=random_regular_graph(10,20,seed)
-        
+
         assert_raises(networkx.exception.NetworkXError,
                       random_regular_graph, 3, 21)
 
@@ -124,5 +118,12 @@ class TestGeneratorsRandom():
         assert_equal(len(G),10)
         assert_equal(len(G.edges()),0)
 
-
-
+    def test_watts_strogatz_big_k(self):
+        assert_raises(networkx.exception.NetworkXError,
+                watts_strogatz_graph, 10, 10, 0.25)
+        assert_raises(networkx.exception.NetworkXError,
+                newman_watts_strogatz_graph, 10, 10, 0.25)
+        # could create an infinite loop, now doesn't
+        # infinite loop used to occur when a node has degree n-1 and needs to rewire
+        watts_strogatz_graph(10, 9, 0.25, seed=0)
+        newman_watts_strogatz_graph(10, 9, 0.5, seed=0)
