@@ -200,8 +200,8 @@ def get_info(dynamic=True):
 
 ## Version information
 name = 'networkx'
-major = 1
-minor = 6
+major = "1"
+minor = "8.1"
 
 
 ## Declare current release as a development release.
