@@ -31,8 +31,8 @@ from setuptools import setup
 if __name__ == "__main__":
 
     setup(
-        name             = release.name,
-        version          = release.version,
+        name             = release.name.lower(),
+        version          = version,
         maintainer       = release.maintainer,
         maintainer_email = release.maintainer_email,
         author           = release.authors['Hagberg'][0],
